<?php

namespace Smsto\Hubspot\Http\Controllers\Api;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Log;
use Smsto\Hubspot\Http\Requests\Api\SmstoRequest;
use Smsto\Hubspot\Http\Requests\Api\SendSmstoRequest;
use Smsto\Hubspot\Lib\UrlHelper;
use Smsto\Hubspot\Models\Settings;
use Illuminate\Support\Str;
use Smsto\Hubspot\Jobs\CreateTimelineJob;
use Smsto\Hubspot\Http\Requests\Api\CallbackSmstoRequest;
use Smsto\Hubspot\Jobs\SendJob;
use Illuminate\Support\Arr;

class SmstoController extends AdminController
{
    /**
     * Get params
     *
     * @author Panayiotis Halouvas <phalouvas@kainotomo.com>
     *
     * @param Smsto\Hubspot\Models\Settings $settings
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function params(Settings $settings)
    {
        $response = [
            "success" => true,
            "message" => null,
            "messages" => null,
            "data" => [
                "show_reports" => $settings->show_reports,
                "show_people" => $settings->show_people
            ]
        ];

        return response($response);
    }

    /**
     * Call smsto
     *
     * @author Panayiotis Halouvas <phalouvas@kainotomo.com>
     *
     * @param Request $request
     * @param Smsto\Hubspot\Models\Settings $settings
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function call(SmstoRequest $request, Settings $settings)
    {
        $validated = $request->validate($request->rules());
        $method = strtolower($validated['_method']);

        $payload = isset($validated['payload']) ? json_decode($validated['payload'], true) : [];
        $url = $validated['_url'];
        if ($method == 'get') {
            $url = $this->sanitizeUrl($url, $payload);
        }

        $response = Http::withHeaders(['X-Smsto-Integration-Name' => 'hubspot'])->withToken($settings->api_key)->asJson()->acceptJson()->$method($url, $payload);
        return response(json_decode($response, true));
    }

    /**
     * Call smsto
     *
     * @author Panayiotis Halouvas <phalouvas@kainotomo.com>
     *
     * @param Request $request
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function send(SendSmstoRequest $request)
    {
        $validated = $request->validated();
        SendJob::dispatch($validated);
        return response('ok');
    }

    /**
     * Get callback of messages status and create timeline
     *
     * @author Panayiotis Halouvas <phalouvas@kainotomo.com>
     *
     * @param Smsto\Hubspot\Http\Requests\Api\CallbackSmstoRequest $callbackSmstoRequest
     * @param Smsto\Hubspot\Models\Settings $settings
     * @param int $associatedObjectId
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function callback(CallbackSmstoRequest $callbackSmstoRequest, Settings $settings, int $associatedObjectId)
    {
        $validated = $callbackSmstoRequest->validated();
        if ($validated['status'] === 'SENT') {
            return response('ok');
        }

        CreateTimelineJob::dispatch($validated, $settings->id, $associatedObjectId)->delay(now()->addMinute());
        return response('ok');
    }


    /**
     * @param string $url
     * @param array $payload
     * @return string
     */
    private function sanitizeUrl(string $url, array &$payload): string
    {
        $parseUrl = parse_url($url);
        if (Arr::has($parseUrl, 'query')) {
            $url = $parseUrl['scheme'] . '://' . $parseUrl['host'].$parseUrl['path'];
            $queryParams = [];
            parse_str($parseUrl['query'], $queryParams);
            $payload = array_merge($payload, $queryParams);
        }

        return $url;
    }
}
